var mongoose = require('mongoose')
 ,  Sensor = mongoose.model('Sensor')
 , eventQueue = require('./event_queue_agent')
 , pubclient = eventQueue.createClient()
 , config = require('../config/config')['development']
 , uuid = require('node-uuid')
 , nodemailer = require('nodemailer')
 , emailer = nodemailer.createTransport("SMTP", {
    service: "Gmail",
      auth: config.email
    });

exports.actions = {
  'toggleThermostat' : function(argArray, cb){
    var id = argArray[0];
    Sensor.findOne({_id: id}, function(err, sensor){
      if(err) return cb(err);
      if(!sensor) return cb();
      else{
        console.log(sensor); 
        var toggleValue = (sensor.lastReading.relay1 == 1) ? '1' : '2';

        var action = { type: 1,
                       user: '512652cba46e449c02000001',
                       mac: '0013a200408a7c91',
                       name: 'set',
                       values: { state: toggleValue, setTemp: '60'}};
        var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
        console.log("pushing action to queue");
        console.log(event);
        pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
        return cb(sensor);
      }
    });
    console.log("toggling thermostat");
  },
  'checkPresence' : function(argArray, user, cb){
    console.log('Checking presence');
    var mac = argArray[0];
    var userid = argArray[1];
    Sensor.findOne({mac:mac, user: userid}, function(err, sensor){
      if(err) return cb(err);
      if(!sensor) return cb(null, false);
      if(!sensor.lastReading) return cb(null, false);
      console.log("Found sensor", sensor);
      return cb(null, sensor.lastReading.inrange);
    });
  },
  'display2Lines' : function(argArray, cb){
    var line1 = argArray[0];
    var line2 = argArray[1];
    var user = argArray[2];
    var mac = argArray[3];
    console.log(line1, line2, user, mac);
    var action = { type :2,
                   user: user,
                   mac: mac,
                   name: 'setString',
                   values: { line1: line1,
                             line2: line2,
                             timeout: '30000'
                           }
                 }
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();

  },
  'sendEmail' : function(argArray, user, cb){
    var to = argArray[0];
    var subject = argArray[1];
    var body = argArray[2];
    var mailOptions = {
      from: "Grid React <gridreact@gmail.com",
      to : to,
      subject: subject,
      text: body
     }
    emailer.sendMail(mailOptions, function(err, emailRes){
       if(err){
         console.log(err);
        }else{
          console.log("message sent ", emailRes.message);
        }
    });
    return cb();
  },
  'playSound' : function(argArray, user,  cb){
    var sound = argArray[0];
    console.log(sound);
    var action = { type :0,
                   user: user,
                   mac: 'sound-driver-gw',
                   name: 'play',
                   values: {
                     sound: sound
                   }
                 };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'gpio' : function(argArray, user,  cb){
    console.log("Running GPIO action");
    var direction = argArray[0];
    var pin = argArray[1];

    var action = { type :0,
                   user: user,
                   mac: 'gpio-driver-gw',
                   name: direction,
                   values: {
                     pin: pin
                   }
                 };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },

  'stopSound' : function(argArray, user,  cb){
    var action = { type :0,
                   user: user,
                   mac: 'sound-driver-gw',
                   name: 'stop',
                   values: {
                   }
                 };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'ringDoorbell' : function(argArray, user ,  cb){
    console.log("Ring door bell " + user);
    var action = {type: 0,
                  user: user,
                  mac: 'sound-driver-gw',
                  name: 'play',
                  values: {
                    sound: 'doorbell.mp3',
                    }
                  };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'blinkLights' : function(argArray, cb){
    var ringing = argArray[0];
    var user = argArray[1];
    var mac = argArray[2];
    var length = 15000; 
    if(!ringing){
      length = 0;
    }
    console.log(ringing, user, mac, length);
    var action = { type :2,
                   user: user,
                   mac: mac,
                   name: 'blink',
                   values: { duration: length,
                            }
                 };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'momentaryDimLight' : function(argArray, cb){
    var user = argArray[0];
    var mac = argArray[1];
    console.log(user, mac);
    var action = { type :2,
                   user: user,
                   mac: mac,
                   name: 'dim',
                   values: {}
                 };
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'sendTextMessage': function(argArray, cb) {
    var destTn = argArray[0];
    var message = argArray[1];
    console.log(destTn, message);

    // call XTM proxy service to deliver text message
    var request = require('request');
    var env = process.env.NODE_ENV || 'development';
    var config = require('../config/config')[env];
    var baseUrl = config.xtmBaseUrl;
    var url = baseUrl + '/api/proxy/xohSend';
    var data = {
      tn: destTn,
      body: message,
      auth_secret: config.xtmAuthSecret
    };
    request.post({url: url, json: true,
             body: data}
             , function(error, response, body) {
      if(!error){
        console.log('sent text message, responseBody=%j', body);
      }else{
        console.log('Error sending text message, error=%s', error);
      }
    });
    
    return cb();
  },
  'displayTemp' : function(argArray, cb){
    var temp = argArray[0];
    var relay = argArray[1];
    var user = argArray[2];
    var mac = argArray[3];
    console.log(temp, relay);
    var currentTime = new Date();
    var relayState = (relay === 1) ? "On" : "Off";
    var action = { type: 2,
                   user: user, 
                   mac: mac,
                   name: 'setString',
                   values: { line1: "temp: " + (new Number(temp)).toPrecision(4) , 
                             line2:"Heating is " + relayState, timeout: '30000'}};
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  },
  'displayCommit' : function(argArray, cb){
    var repo = argArray[0];
    var commit = argArray[1];
    console.log(repo, commit);
    var action = { type: 2,
                   user: '512652cba46e449c02000001',
                   mac: '0013a200407734cf',
                   name: 'setString',
                   values: { line1: repo , line2: commit, timeout: '30000'}};
    var event = { id: uuid.v1(), time: new Date().getTime(), act:action };
    pubclient.rpush("actionQueue", JSON.stringify(event), function(){});
    return cb();
  }
}
