var _ = require('underscore');
var acceptableTimeFrame = 15000;
var ruleDb = require('./rule_db');
var actions = require('./actions').actions;
var async = require('async');

exports.boot = function(handler){
  var jsonrules = require('jsonrules');
  var catalog = new jsonrules.FnCatalog();
  _.forEach(actions, function(fn, name){
    catalog.addFn(fn, name, this);
  });

  var rulesEngine = new jsonrules.JsonRules({catalog: catalog});
  //Checks the time that the event happened so we aren't firing rules for really 
  //old events. This will also help for kicking in auto scale if its ever needed
  //
  var checkTime = function (data, cb){
    var currentTime = new Date().getTime(); 
    if((currentTime - data.time) >= acceptableTimeFrame){
      console.log("don't do action since its old. currentTime =" + 
                  currentTime + " event Time=" + data.time + 
                  " age= " + (currentTime - data.time) + " Figure out what do with it later");
      handler.emit('event::old', data);
      //We throw an error and cancel the waterfall
      return cb(new Error('Old event'), data);
    }else{
      //Otherwise we pass the time test so we can go to the next function
      //and make sure to pass the data object
      return cb(null, data);
    }
  };
  var getRules = function(data, cb){
    ruleDb.getRules(data, function(err, rules, testObj){
      if(rules){
        console.log('Got rules for the event');
        console.log(rules); 
        return cb(null, data, rules, testObj);
      }
      else{
        console.log('No rules for the event');
        return cb(new Error('No rules found'));
      }
    });
  }
  var doRules = function(data, rules, testObj, callback){
    var thenArray = [];
    async.each(rules, function(rule, cb) {

      rulesEngine.doRule(rule, testObj, function(err, thens){
        if(err){
          console.log('received error while performing rule logic', rule, err);
          return cb(new Error('Error with performing rule logic'));
        }
        if(thens){
          thenArray = thenArray.concat(thens);
          return cb(null); 
        } 
        else{
          console.log('rule did not pass no action');
          return cb(null);
        } 
      });

    }, function(err){
        // if any of the doRules produced an error, err would equal that error
        if (err) {
          return callback(new Error('Async Error with performing rule logic'));
        } else {
          console.log('Finished processing all rules, moving onto thens', thenArray);

          callback(null, data, thenArray);
        }
    });

  }
  var filterLowerPriorityThens = function (data, thens, callback){
    if(!thens){
      return callback(null);
    }
    var fnMap = {};
    thens.forEach(function(fn){
      if(fnMap[fn.fnName] && fnMap[fn.fnName] < fn.priority)
        fnMap[fn.fnName] = fn.priority;
      else
        fnMap[fn.fnName] = fn.priority;
    });
    thens = thens.filter(function(fn){
      return fnMap[fn.fnName] <= fn.priority;
    });
    return callback(null, data, thens);
  }
  var doThens = function(data, thens, callback){
    if(!thens){
      return callback(null);
    }

    async.each(thens, function(then, cb){
      then(data.evt.user, function(){
        console.log('finished doing action ' + then.fnName);
        cb(null);
      });
    });

  }

  handler.on('event::new',function(data){
    var injectData = function(cb){
      var closedData = data;
      cb(null, closedData);
    }
    async.waterfall([ injectData, checkTime, getRules, doRules, filterLowerPriorityThens,  doThens], function(err, result){
      if(err){
        handler.emit('event::complete', data);
        console.log(err);
      }else{
        handler.emit('event::complete', data);
        console.log('successfully ran rule');
      }
    });
  });
}

